@@ -1,6 +1,16 @@
+# -*- test-case-name: wokkel.test.test_formats -*-
+#
 # Copyright (c) Ralph Meijer.
 # See LICENSE for details.
 
+"""
+Generic payload formats.
+"""
+
+from __future__ import division, absolute_import
+
+from twisted.python.compat import unicode
+
 NS_MOOD = 'http://jabber.org/protocol/mood'
 NS_TUNE = 'http://jabber.org/protocol/tune'
 
@@ -20,6 +30,7 @@ class Mood:
         self.value = value
         self.text = text
 
+
     def fromXml(self, element):
         """
         Get a Mood instance from an XML representation.
@@ -55,6 +66,8 @@ class Mood:
 
     fromXml = classmethod(fromXml)
 
+
+
 class Tune:
     """
     User tune.
