@@ -7,24 +7,23 @@
 Wokkel interfaces.
 """
 
-__all__ = ['IXMPPHandler', 'IXMPPHandlerCollection',
-           'IPubSubClient', 'IPubSubService', 'IPubSubResource',
-           'IMUCClient', 'IMUCStatuses']
+from __future__ import division, absolute_import
 
 from zope.interface import Interface
 from twisted.python.deprecate import deprecatedModuleAttribute
-from twisted.python.versions import Version
 from twisted.words.protocols.jabber.ijabber import IXMPPHandler
 from twisted.words.protocols.jabber.ijabber import IXMPPHandlerCollection
 
+from incremental import Version
+
 deprecatedModuleAttribute(
-        Version("Wokkel", 0, 7, 0),
+        Version("wokkel", 0, 7, 0),
         "Use twisted.words.protocols.jabber.ijabber.IXMPPHandler instead.",
         __name__,
         "IXMPPHandler")
 
 deprecatedModuleAttribute(
-        Version("Wokkel", 0, 7, 0),
+        Version("wokkel", 0, 7, 0),
         "Use twisted.words.protocols.jabber.ijabber.IXMPPHandlerCollection "
                 "instead.",
         __name__,
@@ -982,3 +981,9 @@ class IMUCStatuses(Interface):
         """
         Return the number of status conditions.
         """
+
+
+
+__all__ = ['IXMPPHandler', 'IXMPPHandlerCollection',
+           'IPubSubClient', 'IPubSubService', 'IPubSubResource',
+           'IMUCClient', 'IMUCStatuses']
