@@ -7,29 +7,31 @@
 XMPP subprotocol support.
 """
 
-__all__ = ['XMPPHandler', 'XMPPHandlerCollection', 'StreamManager',
-           'IQHandlerMixin']
+from __future__ import division, absolute_import
 
-from zope.interface import implements
+from zope.interface import implementer
 
 from twisted.internet import defer
 from twisted.internet.error import ConnectionDone
 from twisted.python import failure, log
+from twisted.python.compat import iteritems, itervalues
 from twisted.python.deprecate import deprecatedModuleAttribute
-from twisted.python.versions import Version
 from twisted.words.protocols.jabber import error, ijabber, xmlstream
 from twisted.words.protocols.jabber.xmlstream import toResponse
 from twisted.words.protocols.jabber.xmlstream import XMPPHandlerCollection
 from twisted.words.xish import xpath
 from twisted.words.xish.domish import IElement
 
+from incremental import Version
+
 deprecatedModuleAttribute(
-        Version("Wokkel", 0, 7, 0),
+        Version("wokkel", 0, 7, 0),
         "Use twisted.words.protocols.jabber.xmlstream.XMPPHandlerCollection "
                 "instead.",
         __name__,
         "XMPPHandlerCollection")
 
+@implementer(ijabber.IXMPPHandler)
 class XMPPHandler(object):
     """
     XMPP protocol handler.
@@ -38,8 +40,6 @@ class XMPPHandler(object):
     extension protocols, and are referred to as a subprotocol implementation.
     """
 
-    implements(ijabber.IXMPPHandler)
-
     def __init__(self):
         self.parent = None
         self.xmlstream = None
@@ -277,7 +277,7 @@ class StreamManager(XMPPHandlerCollection):
         # deferreds will never be fired.
         iqDeferreds = self._iqDeferreds
         self._iqDeferreds = {}
-        for d in iqDeferreds.itervalues():
+        for d in itervalues(iqDeferreds):
             d.errback(reason)
 
 
@@ -455,7 +455,7 @@ class IQHandlerMixin(object):
             return error.StanzaError('internal-server-error').toResponse(iq)
 
         handler = None
-        for queryString, method in self.iqHandlers.iteritems():
+        for queryString, method in iteritems(self.iqHandlers):
             if xpath.internQuery(queryString).matches(iq):
                 handler = getattr(self, method)
 
@@ -472,3 +472,8 @@ class IQHandlerMixin(object):
         d.addCallback(self.send)
 
         iq.handled = True
+
+
+
+__all__ = ['XMPPHandler', 'XMPPHandlerCollection', 'StreamManager',
+           'IQHandlerMixin']
