@@ -5,6 +5,8 @@
 Tests for L{wokkel.muc}
 """
 
+from __future__ import division, absolute_import
+
 from datetime import datetime
 from dateutil.tz import tzutc
 
@@ -12,6 +14,7 @@ from zope.interface import verify
 
 from twisted.trial import unittest
 from twisted.internet import defer, task
+from twisted.python.compat import iteritems, unicode
 from twisted.words.xish import domish, xpath
 from twisted.words.protocols.jabber.jid import JID
 from twisted.words.protocols.jabber.error import StanzaError
@@ -78,7 +81,7 @@ class StatusCodeTest(unittest.TestCase):
             332: 'removed-shutdown',
         }
 
-        for code, condition in codes.iteritems():
+        for code, condition in iteritems(codes):
             constantName = condition.replace('-', '_').upper()
             self.assertEqual(getattr(muc.STATUS_CODE, constantName),
                              muc.STATUS_CODE.lookupByValue(code))
@@ -823,7 +826,7 @@ class MUCClientProtocolTest(unittest.TestCase):
         nodes = xpath.queryForNodes(query, iq)
         self.assertNotIdentical(None, nodes, 'Missing query element')
 
-        self.assertRaises(StopIteration, nodes[0].elements().next)
+        self.assertRaises(StopIteration, next, nodes[0].elements())
 
         xml = u"""
             <iq from='%s' id='%s' to='%s' type='result'>
@@ -969,7 +972,7 @@ class MUCClientProtocolTest(unittest.TestCase):
         nodes = xpath.queryForNodes(query, iq)
         self.assertNotIdentical(None, nodes, 'Missing query element')
 
-        self.assertRaises(StopIteration, nodes[0].elements().next)
+        self.assertRaises(StopIteration, next, nodes[0].elements())
 
         xml = u"""
             <iq from='%s' id='%s' to='%s' type='result'>
@@ -1390,7 +1393,7 @@ class MUCClientProtocolTest(unittest.TestCase):
 
         iq = self.stub.output[-1]
 
-        query = ("/iq/query[@xmlns='%s']/destroy[@xmlns='%s']" %
+        query = ("/iq[@type='set']/query[@xmlns='%s']/destroy[@xmlns='%s']" %
                  (muc.NS_MUC_OWNER, muc.NS_MUC_OWNER))
 
         nodes = xpath.queryForNodes(query, iq)
@@ -1490,6 +1493,47 @@ class MUCClientTest(unittest.TestCase):
         self._testPresence(sender=otherOccupantJID)
 
 
+    def test_availableReceivedSetsUserRole(self):
+        """
+        The role received in a presence update is stored on the user.
+        """
+        room = self._createRoom()
+        user = muc.User(self.nick)
+        room.addUser(user)
+        self.assertEquals('none', user.role)
+
+        xml = u"""
+            <presence to='%s' from='%s'>
+              <x xmlns='http://jabber.org/protocol/muc#user'>
+                <item affiliation='member' role='participant'/>
+              </x>
+            </presence>
+        """ % (self.userJID, self.occupantJID)
+        self.stub.send(parseXml(xml))
+
+        self.assertEquals('participant', user.role)
+
+
+    def test_availableReceivedSetsUserAffiliation(self):
+        """
+        The affiliation received in a presence update is stored on the user.
+        """
+        room = self._createRoom()
+        user = muc.User(self.nick)
+        room.addUser(user)
+        self.assertEquals('none', user.affiliation)
+
+        xml = u"""
+            <presence to='%s' from='%s'>
+              <x xmlns='http://jabber.org/protocol/muc#user'>
+                <item affiliation='member' role='participant'/>
+              </x>
+            </presence>
+        """ % (self.userJID, self.occupantJID)
+        self.stub.send(parseXml(xml))
+        self.assertEquals('member', user.affiliation)
+
+
     def test_unavailableReceivedEmptySender(self):
         """
         Availability presence from empty sender is ignored.
